/*****************************************************************************/
/*
                              connect_example.c

This small program is used to test and exercise the [ServiceConnect] mode of
"request" processing.  Assumes the client program is a TELNET application.  If
the mapping contains a translated path then output the content of that file. 
Otherwise, output selected CGI variables and lines of input to the client
TELNET session.


# WASD_CONFIG_SERVICE
[[*:23]]
[ServiceConnect] enabled

# WASD_CONFIG_MAP
[[*:23]]
###pass * /dka100/wasd/port23.txt response=var=crlf
exec /cgi-bin/* /cgi-bin/* map=once
###redirect * /cgi-bin/connect_example
redirect * /cgi-bin/connect_example/wasd_root/readme.html


BUILD DETAILS
-------------
$ @BUILD_CONNECT_EXAMPLE BUILD
$ @BUILD_CONNECT_EXAMPLE LINK


COPYRIGHT
---------
Copyright (C) 2021 Mark G.Daniel

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.


VERSION HISTORY
---------------
22-FEB-2021  MGD  v1.0.0, initial development
*/

/*****************************************************************************/

/* standard C header files */
#include <ctype.h>
#include <errno.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

char* CgiVar (char*);

/*****************************************************************************/
/*
*/

main (int argc, char* argv[])
       
{
   int  cnt = 0;
   char  *cptr;
   char  record [256];
   FILE  *fptr;

   /*********/
   /* begin */
   /*********/

   fputs ("Content-Type: text/plain\r\n", stdout);
   fputs ("\r\n", stdout);

   if (*(cptr = CgiVar("WWW_PATH_TRANSLATED")))
   {
      /* there is a translated path so output the file content */
      if (!(fptr = fopen (cptr, "r")))
      {
         fprintf (stdout, "%s %s\n", cptr, strerror(errno));
         exit (1);
      }
      for (;;)
      {
         if (!fgets (record, sizeof(record), fptr)) break;
         fprintf (stdout, "%d. %s", ++cnt, record);
      }
   }
   else
   {
      /* no translated path - output selected CGI variables and TELNET input */
      fprintf (stdout, "%s=%s\n", cptr = "WWW_SERVER_SOFTWARE", CgiVar(cptr));
      fprintf (stdout, "%s=%s\n", cptr = "WWW_CONTENT_LENGTH", CgiVar(cptr));
      fprintf (stdout, "%s=%s\n", cptr = "WWW_CONTENT_TYPE", CgiVar(cptr));
      fprintf (stdout, "%s=%s\n", cptr = "WWW_PATH_INFO", CgiVar(cptr));
      fprintf (stdout, "%s=%s\n", cptr = "WWW_PATH_TRANSLATED", CgiVar(cptr));
      fprintf (stdout, "%s=%s\n", cptr = "WWW_QUERY_STRING", CgiVar(cptr));
      fprintf (stdout, "%s=%s\n", cptr = "WWW_SCRIPT_FILENAME", CgiVar(cptr));
      fprintf (stdout, "%s=%s\n", cptr = "WWW_SCRIPT_NAME", CgiVar(cptr));
      fprintf (stdout, "%s=%s\n", cptr = "WWW_SERVER_NAME", CgiVar(cptr));
      fprintf (stdout, "%s=%s\n", cptr = "WWW_SERVER_PORT", CgiVar(cptr));

      if (!(fptr = fopen ("HTTP$INPUT", "r")))
      {
         fprintf (stdout, "%s\n", strerror(errno));
         exit (1);
      }

      fprintf (stdout, "%s\n", CgiVar("WWW_GATEWAY_ESC"));
      fprintf (stdout, "#CLIENT-READ:\n");
      fprintf (stdout, "%s\n", CgiVar("WWW_GATEWAY_EOT"));

      /* assume it's telnet and discard the first record */
      fgets (record, sizeof(record), fptr);
      for (;;)
      {
         fgets (record, sizeof(record), fptr);
         if (record[0] == '\n') continue;
         if (record[0] == '\r') break;
         fprintf (stdout, "%d. %s", ++cnt, record);
      }
   }
}

/*****************************************************************************/
/*
Return the value of the supplied CGI variable name, or NULL.
*/

char* CgiVar (char *name)
       
{
   static int  www = -1;

   /*********/
   /* begin */
   /*********/

   if (www < 0)
   {
      if (getenv ("WWW_SERVER_SOFTWARE"))
         www = 1;
      else
      if (getenv ("SERVER_SOFTWARE"))
         www = 0;
      else
         exit (676);
   }

   if (www && !strncasecmp(name, "WWW_", 4)) return (getenv (name));
   if (!www && strncasecmp(name, "WWW_", 4)) return (getenv (name));
   return (getenv (name+4));
}

/*****************************************************************************/

